import { BaseProps } from '../../base/props';
import { PickerOption, PickerOptions, PickerValue, BasePicker } from '../picker/base';
export interface BaseDatePicker extends BaseProps {
    /**
    * 受控
    */
    value?: Date
    /**
    * 初始值
    */
    defaultValue?: Date
    /**
    * 是否可见
    */
    visible: boolean
    /**
    * 设置标题
    */
    title: string
    /**
    * 类时间类型，可选值 date time year-month month-day datehour datetime hour-minutes
    */
    type: 'date' | 'time' | 'year-month' | 'month-day' | 'datehour' | 'datetime' | 'hour-minutes'
    /**
    * 每列是否展示中文
    */
    showChinese: boolean
    /**
    * 分钟步进值
    */
    minuteStep: number
    /**
    * 开始日期
    */
    startDate: Date
    /**
    * 结束日期
    */
    endDate: Date
    /**
    * 是否开启3D效果
    */
    threeDimensional: boolean
    /**
    * 透传picker属性
    */
    pickerProps: Partial<Omit<BasePicker, 'defaultValue' | 'threeDimensional' | 'title' | 'value' | 'onConfirm' | 'onClose' | 'onCancel' | 'onChange'>>
    /**
    * 选项格式化函数
    */
    formatter: (type: string, option: PickerOption) => PickerOption
    /**
    * 选项过滤函数
    */
    filter: (type: string, options: PickerOptions) => PickerOptions
    /**
    * 确定和取消时，都触发
    */
    onClose: () => void
    /**
    * 点击取消按钮时触发
    */
    onCancel: () => void
    /**
    * 点击确定按钮时触发
    */
    onConfirm: (selectedOptions: PickerOptions, selectedValue: PickerValue[]) => void
    /**
    * 选项改变时触发
    */
    onChange?: (selectedOptions: PickerOptions, selectedValue: PickerValue[], columnIndex: number) => void
    children?: any;
}
