import { ReactNode } from 'react';
import { BaseProps } from '../../base/props';
import { FixAutoComplete } from '../../helpers/fix-string-literal-union';
import { Align, PositionY, SimpleValue } from '../../base/atoms';
export type ImageFit = FixAutoComplete<'contain' | 'cover' | 'fill' | 'none' | 'scale-down'>;
export type ImagePosition = FixAutoComplete<Align | PositionY>;
export interface BaseImage extends BaseProps {
    /**
    * 图片链接
    */
    src: string
    /**
    * 图片填充模式，等同于原生的 object-fit 属性
    */
    fit: ImageFit
    /**
    * 图片位置，等同于原生的 object-position 属性
    */
    position: ImagePosition
    /**
    * 替代文本
    */
    alt: string
    /**
    * 宽度，默认单位px
    */
    width: string | number
    /**
    * 高度，默认单位px
    */
    height: string | number
    /**
    * 圆角大小
    */
    radius: SimpleValue
    /**
    * 是否展示图片加载失败
    */
    error: ReactNode
    /**
    * 是否展示加载中图片
    */
    loading: ReactNode
    /**
    * 是否为懒加载图片
    */
    lazy: boolean
    draggable: boolean;
    /**
    * 图片加载完后触发
    */
    onLoad: () => void
    /**
    * 图片加载失败后触发
    */
    onError: () => void
    /**
    * 点击图片时触发
    */
    onClick: (e: any) => void
}
