import { ReactNode } from 'react';
import { EdgePosition, PositionY, SimpleValue } from '../../base/atoms';
import { BaseProps } from '../../base/props';
export interface PreviewImageOption {
    src: string;
    index?: number;
}
export interface PreviewVideoOption {
    source: {
        src: string;
        type: string;
    };
    options: {
        muted: boolean;
        controls: boolean;
    };
    index?: number;
}
export type ImagePreviewCloseIconPosition = Extract<EdgePosition, 'top-right' | 'top-left'> | Extract<PositionY, 'bottom'>;
export interface BaseImagePreview<IMAGE_SOURCE = PreviewImageOption, VIDEO_SOURCE = PreviewVideoOption> extends BaseProps {
    /**
    * 预览图片数组
    */
    images: Array<IMAGE_SOURCE>
    /**
    * 预览的视频数组（视频自动放到图片之前）
    */
    videos: Array<VIDEO_SOURCE>
    /**
    * 是否展示预览图片
    */
    visible: boolean
    /**
    * 自动轮播时长，0表示不会自动轮播
    */
    autoPlay: SimpleValue
    /**
    * 页码，受控
    */
    value?: number
    /**
    * 初始页码
    */
    defaultValue: number
    /**
    * 点击图片可以退出预览
    */
    closeOnContentClick: boolean
    /**
    * 分页是否展示
    */
    pagination: boolean
    /**
    * 分页指示器是否展示
    */
    indicator: boolean
    /**
    * 分页指示器选中的颜色
    */
    indicatorColor: string
    /**
    * 关闭按钮
    */
    closeIcon: ReactNode
    /**
    * 关闭按钮位置
    */
    closeIconPosition: ImagePreviewCloseIconPosition
    /**
    * 点击遮罩关闭图片预览时触发
    */
    onClose: () => void
    /**
    * 切换时触发
    */
    onChange: (value: number) => void
}
