import { ReactNode } from 'react';
import { BaseProps } from '../../base/props';
import { UIType } from '../../base/atoms';
export type PullToRefreshType = Extract<UIType, 'default' | 'primary'>;
export type PullStatus = 'pulling' | 'canRelease' | 'refreshing' | 'complete';
export interface BasePullToRefresh extends BaseProps {
    /**
    * 触发刷新时的处理函数
    */
    onRefresh: () => Promise<any>
    type: PullToRefreshType;
    /**
    * 下拉的提示文案
    */
    pullingText: ReactNode
    /**
    * 释放的提示文案
    */
    canReleaseText: ReactNode
    /**
    * 刷新时的提示文案
    */
    refreshingText: ReactNode
    /**
    * 完成时的提示文案
    */
    completeText: ReactNode
    /**
    * 完成后延迟消失的时间，单位为 ms
    */
    completeDelay: number
    /**
    * 头部提示内容区的高度，单位为 px
    */
    headHeight: number
    /**
    * 触发刷新需要下拉多少距离，单位为 px
    */
    threshold: number
    /**
    * 是否禁用下拉刷新
    */
    disabled: boolean
    /**
    * 根据下拉状态，自定义下拉提示图标
    */
    renderIcon: (status: PullStatus) => ReactNode
    /**
    * 根据下拉状态，自定义下拉提示文案
    */
    renderText: (status: PullStatus) => ReactNode
}
