import { ReactNode } from 'react';
import { BaseProps } from '../../base/props';
import { PositionX, SimpleValue } from '../../base/atoms';
export interface SwipeRef {
    open: (side: PositionX) => void;
    close: () => void;
}
export interface BaseSwipe extends BaseProps {
    /**
    * 标识符，可以在事件参数中获取到
    */
    name?: SimpleValue
    /**
    * 左侧滑动区域的内容
    */
    leftAction?: ReactNode
    /**
    * 右侧滑动区域的内容
    */
    rightAction?: ReactNode
    /**
    * 关闭前的回调函数，返回滑动区域所在方向 position
    */
    beforeClose?: (position: string) => void;
    /**
    * 是否禁用滑动
    */
    disabled?: boolean
    /**
    * 打开单元格侧边栏
    */
    onOpen?: ({ name, position, }: {
        name: string | number;
        position: PositionX;
    }) => void
    /**
    * 收起单元格侧边栏
    */
    onClose?: ({ name, position, }: {
        name: string | number;
        position: PositionX;
    }) => void
    /**
    * 点击左侧或者右侧时触发
    */
    onActionClick?: (event: any, position: PositionX) => void
    /**
    * 开始触碰时触发
    */
    onTouchStart?: (event: any) => void
    /**
    * 结束触碰时触发
    */
    onTouchEnd?: (event: any) => void
    /**
    * 滑动时触发
    */
    onTouchMove?: (event: any) => void
}
