import { ReactNode } from 'react';
import { BaseProps } from '../../base/props';
import { Direction, SimpleValue } from '../../base/atoms';
export type SwiperRef = {
    to: (index: number) => void;
    next: () => void;
    prev: () => void;
};
export interface FocusEffect {
    name: 'focus';
    scale: number;
}
export interface BaseSwiper extends BaseProps {
    /**
    * 轮播方向
    */
    direction: Direction
    /**
    * 分页指示器是否展示，可传入自定义的 HTML 结构
    */
    indicator: ReactNode
    /**
    * 是否循环轮播
    */
    loop: boolean
    /**
    * 动画时长（单位是ms）
    */
    duration: SimpleValue
    /**
     * @deprecated Please use `autoplay` prop instead.
     */
    autoPlay: boolean | number;
    /**
    * 自动轮播时长，0表示不会自动轮播
    */
    autoplay: boolean | number
    /**
    * 初始化索引值
    */
    defaultValue: number
    /**
    * 是否可触摸滑动
    */
    touchable: boolean
    effect: FocusEffect | undefined;
    /**
    * 轮播卡片的尺寸
    */
    slideSize?: number
    /**
    * 卡片切换后的回调
    */
    onChange?: (index: number) => void
}
export interface BaseSwiperItem<Event = any> extends BaseProps {
    onClick?: (e: Event) => void;
}
