import { CSSProperties, ReactNode } from 'react';
import { BaseProps } from '../../base/props';
import { Direction, PositionX, SimpleValue } from '../../base/atoms';
export type TabsTitle = {
    title: string;
    disabled: boolean;
    active?: boolean;
    value: SimpleValue;
};
export interface BaseTabs extends BaseProps {
    /**
    * 标签栏样式
    */
    tabStyle: CSSProperties
    /**
    * 当前激活 tab 面板的值
    */
    value: SimpleValue
    /**
    * 初始化激活 tab 的值
    */
    defaultValue: SimpleValue
    /**
    * 标签选中色
    */
    activeColor: string
    /**
    * 使用横纵方向
    */
    direction: Direction
    /**
    * 选中底部展示样式 可选值 line、smile、simple、card、button、divider
    */
    activeType: 'line' | 'smile' | 'simple' | 'card' | 'button' | 'divider'
    /**
    * 切换动画时长,单位 ms 0 代表无动画
    */
    duration: number | string
    /**
    * 标题对齐方式
    */
    align: PositionX
    /**
    * 自定义导航区域
    */
    title: () => ReactNode[]
    /**
    * 当前激活的标签改变时触发
    */
    onChange: (index: SimpleValue) => void
    /**
    * 点击标签时触发
    */
    onClick: (index: SimpleValue) => void
    /**
    * 自动高度。设置为 true 时，nut-tabs 和 nut-tabs\_\_content 会随着当前 nut-tabpane 的高度而发生变化。
    */
    autoHeight: boolean
}
export interface BaseTabPane extends BaseProps {
    title: SimpleValue;
    value: SimpleValue;
    disabled: boolean;
}
