import { ReactNode } from 'react';
import { BaseProps } from '../../base/props';
import { FullPosition, SimpleValue } from '../../base/atoms';
export interface TourList {
    target: Element | string;
    content?: string;
    location?: FullPosition;
    popoverOffset?: number[];
    arrowOffset?: number;
}
export type TourType = 'step' | 'tile';
export interface BaseTour extends BaseProps {
    /**
    * 是否展示引导弹出层
    */
    visible: boolean
    /**
    * 引导类型
    */
    type: TourType
    /**
    * 弹出层位置,同 Popopver 的[location 属性](#/zh-CN/component/popover)
    */
    location: FullPosition
    /**
    * 是否显示镂空遮罩
    */
    mask: boolean
    /**
    * 镂空遮罩层宽度
    */
    maskWidth: SimpleValue
    /**
    * 镂空遮罩层高度
    */
    maskHeight: SimpleValue
    /**
    * 镂空遮罩相对于目标元素的偏移量
    */
    offset: number[]
    /**
    * 引导步骤内容
    */
    list: TourList[]
    /**
    * 是否展示标题栏
    */
    title: ReactNode
    /**
    * 下一步按钮文案
    */
    next: ReactNode
    /**
    * 上一步按钮文案
    */
    prev: ReactNode
    /**
    * 完成按钮文案
    */
    complete: ReactNode
    /**
    * 是否展示上一步按钮
    */
    showPrev: boolean
    /**
    * 是否在点击镂空遮罩层后关闭,同 Popopver 的[closeOnClickOverlay 属性](#/zh-CN/component/popover)
    */
    closeOnOverlayClick: boolean
    /**
    * 气泡层关闭时触发
    */
    onClose: (e: any) => void
    /**
    * 切换步骤时触发
    */
    onChange: (value: number) => void
}
